@@ -4,9 +4,9 @@
  * gimpsettingsbox.c
  * Copyright (C) 2008 Michael Natterer <mitch@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -34,6 +33,7 @@
 #include "core/gimpmarshal.h"
 
 #include "gimpcontainercombobox.h"
+#include "gimpcontainertreestore.h"
 #include "gimpcontainerview.h"
 #include "gimpsettingsbox.h"
 #include "gimpsettingseditor.h"
@@ -60,9 +60,35 @@ enum
 };
 
 
-static GObject * gimp_settings_box_constructor   (GType              type,
-                                                  guint              n_params,
-                                                  GObjectConstructParam *params);
+typedef struct _GimpSettingsBoxPrivate GimpSettingsBoxPrivate;
+
+struct _GimpSettingsBoxPrivate
+{
+  GtkWidget     *combo;
+  GtkWidget     *menu;
+  GtkWidget     *import_item;
+  GtkWidget     *export_item;
+  GtkWidget     *file_dialog;
+  GtkWidget     *editor_dialog;
+
+  Gimp          *gimp;
+  GObject       *config;
+  GimpContainer *container;
+  gchar         *filename;
+
+  gchar         *import_dialog_title;
+  gchar         *export_dialog_title;
+  gchar         *file_dialog_help_id;
+  gchar         *default_folder;
+  gchar         *last_filename;
+};
+
+#define GET_PRIVATE(item) G_TYPE_INSTANCE_GET_PRIVATE (item, \
+                                                       GIMP_TYPE_SETTINGS_BOX, \
+                                                       GimpSettingsBoxPrivate)
+
+
+static void      gimp_settings_box_constructed   (GObject           *object);
 static void      gimp_settings_box_finalize      (GObject           *object);
 static void      gimp_settings_box_set_property  (GObject           *object,
                                                   guint              property_id,
@@ -114,9 +140,11 @@ static void  gimp_settings_box_manage_response   (GtkWidget         *widget,
                                                   GimpSettingsBox   *box);
 static void  gimp_settings_box_toplevel_unmap    (GtkWidget         *toplevel,
                                                   GtkWidget         *dialog);
+static void  gimp_settings_box_truncate_list     (GimpSettingsBox   *box,
+                                                  gint               max_recent);
 
 
-G_DEFINE_TYPE (GimpSettingsBox, gimp_settings_box, GTK_TYPE_HBOX)
+G_DEFINE_TYPE (GimpSettingsBox, gimp_settings_box, GTK_TYPE_BOX)
 
 #define parent_class gimp_settings_box_parent_class
 
@@ -159,7 +187,7 @@ gimp_settings_box_class_init (GimpSettingsBoxClass *klass)
                   G_TYPE_BOOLEAN, 1,
                   G_TYPE_STRING);
 
-  object_class->constructor  = gimp_settings_box_constructor;
+  object_class->constructed  = gimp_settings_box_constructed;
   object_class->finalize     = gimp_settings_box_finalize;
   object_class->set_property = gimp_settings_box_set_property;
   object_class->get_property = gimp_settings_box_get_property;
@@ -195,60 +223,63 @@ gimp_settings_box_class_init (GimpSettingsBoxClass *klass)
                                                         NULL,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
+
+  g_type_class_add_private (klass, sizeof (GimpSettingsBoxPrivate));
 }
 
 static void
 gimp_settings_box_init (GimpSettingsBox *box)
 {
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (box),
+                                  GTK_ORIENTATION_HORIZONTAL);
+
   gtk_box_set_spacing (GTK_BOX (box), 6);
 }
 
-static GObject *
-gimp_settings_box_constructor (GType                  type,
-                               guint                  n_params,
-                               GObjectConstructParam *params)
+static void
+gimp_settings_box_constructed (GObject *object)
 {
-  GObject         *object;
-  GimpSettingsBox *box;
-  GtkWidget       *hbox2;
-  GtkWidget       *button;
-  GtkWidget       *image;
-  GtkWidget       *arrow;
+  GimpSettingsBox        *box     = GIMP_SETTINGS_BOX (object);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (object);
+  GtkWidget              *hbox2;
+  GtkWidget              *button;
+  GtkWidget              *image;
+  GtkWidget              *arrow;
 
-  object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
+  if (G_OBJECT_CLASS (parent_class)->constructed)
+    G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  box = GIMP_SETTINGS_BOX (object);
+  g_assert (GIMP_IS_GIMP (private->gimp));
+  g_assert (GIMP_IS_CONFIG (private->config));
+  g_assert (GIMP_IS_CONTAINER (private->container));
+  g_assert (private->filename != NULL);
 
-  g_assert (GIMP_IS_GIMP (box->gimp));
-  g_assert (GIMP_IS_CONFIG (box->config));
-  g_assert (GIMP_IS_CONTAINER (box->container));
-  g_assert (box->filename != NULL);
-
-  if (gimp_container_num_children (box->container) == 0)
+  if (gimp_container_get_n_children (private->container) == 0)
     gimp_settings_box_deserialize (box);
 
-  box->combo = gimp_container_combo_box_new (box->container,
-                                             gimp_get_user_context (box->gimp),
-                                             16, 0);
-  gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (box->combo),
+  private->combo = gimp_container_combo_box_new (private->container,
+                                                 gimp_get_user_context (private->gimp),
+                                                 16, 0);
+  gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (private->combo),
                                         gimp_settings_box_row_separator_func,
                                         NULL, NULL);
-  gtk_box_pack_start (GTK_BOX (box), box->combo, TRUE, TRUE, 0);
-  gtk_widget_show (box->combo);
+  gtk_box_pack_start (GTK_BOX (box), private->combo, TRUE, TRUE, 0);
+  gtk_widget_show (private->combo);
 
-  gimp_help_set_help_data (box->combo, _("Pick a setting from the list"),
+  gimp_help_set_help_data (private->combo, _("Pick a setting from the list"),
                            NULL);
 
-  g_signal_connect_after (box->combo, "select-item",
+  g_signal_connect_after (private->combo, "select-item",
                           G_CALLBACK (gimp_settings_box_setting_selected),
                           box);
 
-  hbox2 = gtk_hbox_new (TRUE, 0);
+  hbox2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (hbox2), TRUE);
   gtk_box_pack_start (GTK_BOX (box), hbox2, FALSE, FALSE, 0);
   gtk_widget_show (hbox2);
 
   button = gtk_button_new ();
-  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (button, FALSE);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
@@ -264,7 +295,7 @@ gimp_settings_box_constructor (GType                  type,
                     box);
 
   button = gtk_button_new ();
-  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (button, FALSE);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   gtk_box_pack_start (GTK_BOX (hbox2), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
@@ -279,16 +310,16 @@ gimp_settings_box_constructor (GType                  type,
 
   /*  Favorites menu  */
 
-  box->menu = gtk_menu_new ();
-  gtk_menu_attach_to_widget (GTK_MENU (box->menu), button, NULL);
+  private->menu = gtk_menu_new ();
+  gtk_menu_attach_to_widget (GTK_MENU (private->menu), button, NULL);
 
-  box->import_item =
+  private->import_item =
     gimp_settings_box_menu_item_add (box,
                                      GTK_STOCK_OPEN,
                                      _("_Import Settings from File..."),
                                      G_CALLBACK (gimp_settings_box_import_activate));
 
-  box->export_item =
+  private->export_item =
     gimp_settings_box_menu_item_add (box,
                                      GTK_STOCK_SAVE,
                                      _("_Export Settings to File..."),
@@ -300,61 +331,59 @@ gimp_settings_box_constructor (GType                  type,
                                    GTK_STOCK_EDIT,
                                    _("_Manage Settings..."),
                                    G_CALLBACK (gimp_settings_box_manage_activate));
-
-  return object;
 }
 
 static void
 gimp_settings_box_finalize (GObject *object)
 {
-  GimpSettingsBox *box = GIMP_SETTINGS_BOX (object);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (object);
 
-  if (box->config)
+  if (private->config)
     {
-      g_object_unref (box->config);
-      box->config = NULL;
+      g_object_unref (private->config);
+      private->config = NULL;
     }
 
-  if (box->container)
+  if (private->container)
     {
-      g_object_unref (box->container);
-      box->container = NULL;
+      g_object_unref (private->container);
+      private->container = NULL;
     }
 
-  if (box->filename)
+  if (private->filename)
     {
-      g_free (box->filename);
-      box->filename = NULL;
+      g_free (private->filename);
+      private->filename = NULL;
     }
 
-  g_free (box->import_dialog_title);
-  g_free (box->export_dialog_title);
-  g_free (box->file_dialog_help_id);
-  g_free (box->default_folder);
-  g_free (box->last_filename);
+  g_free (private->import_dialog_title);
+  g_free (private->export_dialog_title);
+  g_free (private->file_dialog_help_id);
+  g_free (private->default_folder);
+  g_free (private->last_filename);
 
-  if (box->editor_dialog)
+  if (private->editor_dialog)
     {
-      GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (box));
+      GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (object));
 
       if (toplevel)
         g_signal_handlers_disconnect_by_func (toplevel,
                                               gimp_settings_box_toplevel_unmap,
-                                              box->editor_dialog);
+                                              private->editor_dialog);
 
-      gtk_widget_destroy (box->editor_dialog);
+      gtk_widget_destroy (private->editor_dialog);
     }
 
-  if (box->file_dialog)
+  if (private->file_dialog)
     {
-      GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (box));
+      GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (object));
 
       if (toplevel)
         g_signal_handlers_disconnect_by_func (toplevel,
                                               gimp_settings_box_toplevel_unmap,
-                                              box->file_dialog);
+                                              private->file_dialog);
 
-      gtk_widget_destroy (box->file_dialog);
+      gtk_widget_destroy (private->file_dialog);
     }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -366,24 +395,24 @@ gimp_settings_box_set_property (GObject      *object,
                                 const GValue *value,
                                 GParamSpec   *pspec)
 {
-  GimpSettingsBox *box = GIMP_SETTINGS_BOX (object);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_GIMP:
-      box->gimp = g_value_get_object (value); /* don't dup */
+      private->gimp = g_value_get_object (value); /* don't dup */
       break;
 
     case PROP_CONFIG:
-      box->config = g_value_dup_object (value);
+      private->config = g_value_dup_object (value);
       break;
 
     case PROP_CONTAINER:
-      box->container = g_value_dup_object (value);
+      private->container = g_value_dup_object (value);
       break;
 
     case PROP_FILENAME:
-      box->filename = g_value_dup_string (value);
+      private->filename = g_value_dup_string (value);
       break;
 
    default:
@@ -398,24 +427,24 @@ gimp_settings_box_get_property (GObject    *object,
                                 GValue     *value,
                                 GParamSpec *pspec)
 {
-  GimpSettingsBox *box = GIMP_SETTINGS_BOX (object);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_GIMP:
-      g_value_set_object (value, box->gimp);
+      g_value_set_object (value, private->gimp);
       break;
 
     case PROP_CONFIG:
-      g_value_set_object (value, box->config);
+      g_value_set_object (value, private->config);
       break;
 
     case PROP_CONTAINER:
-      g_value_set_object (value, box->container);
+      g_value_set_object (value, private->container);
       break;
 
     case PROP_FILENAME:
-      g_value_set_string (value, box->filename);
+      g_value_set_string (value, private->filename);
       break;
 
    default:
@@ -427,7 +456,8 @@ gimp_settings_box_get_property (GObject    *object,
 static void
 gimp_settings_box_separator_add (GimpContainer *container)
 {
-  GimpObject *sep = g_object_new (container->children_type, NULL);
+  GimpObject *sep = g_object_new (gimp_container_get_children_type (container),
+                                  NULL);
 
   gimp_container_add (container, sep);
   g_object_unref (sep);
@@ -448,47 +478,49 @@ gimp_settings_box_separator_remove (GimpContainer *container)
 static void
 gimp_settings_box_deserialize (GimpSettingsBox *box)
 {
-  GError *error = NULL;
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GError                 *error   = NULL;
 
-  if (box->gimp->be_verbose)
-    g_print ("Parsing '%s'\n", gimp_filename_to_utf8 (box->filename));
+  if (private->gimp->be_verbose)
+    g_print ("Parsing '%s'\n", gimp_filename_to_utf8 (private->filename));
 
-  if (! gimp_config_deserialize_file (GIMP_CONFIG (box->container),
-                                      box->filename,
+  if (! gimp_config_deserialize_file (GIMP_CONFIG (private->container),
+                                      private->filename,
                                       NULL, &error))
     {
       if (error->code != GIMP_CONFIG_ERROR_OPEN_ENOENT)
-        gimp_message (box->gimp, NULL, GIMP_MESSAGE_ERROR,
-                      "%s", error->message);
+        gimp_message_literal (private->gimp, NULL, GIMP_MESSAGE_ERROR,
+			      error->message);
 
       g_clear_error (&error);
     }
 
-  gimp_settings_box_separator_add (box->container);
+  gimp_settings_box_separator_add (private->container);
 }
 
 static void
 gimp_settings_box_serialize (GimpSettingsBox *box)
 {
-  GError *error = NULL;
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GError                 *error   = NULL;
 
-  gimp_settings_box_separator_remove (box->container);
+  gimp_settings_box_separator_remove (private->container);
 
-  if (box->gimp->be_verbose)
-    g_print ("Writing '%s'\n", gimp_filename_to_utf8 (box->filename));
+  if (private->gimp->be_verbose)
+    g_print ("Writing '%s'\n", gimp_filename_to_utf8 (private->filename));
 
-  if (! gimp_config_serialize_to_file (GIMP_CONFIG (box->container),
-                                       box->filename,
+  if (! gimp_config_serialize_to_file (GIMP_CONFIG (private->container),
+                                       private->filename,
                                        "settings",
                                        "end of settings",
                                        NULL, &error))
     {
-      gimp_message (box->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "%s", error->message);
+      gimp_message_literal (private->gimp, NULL, GIMP_MESSAGE_ERROR,
+			    error->message);
       g_clear_error (&error);
     }
 
-  gimp_settings_box_separator_add (box->container);
+  gimp_settings_box_separator_add (private->container);
 }
 
 static GtkWidget *
@@ -497,7 +529,8 @@ gimp_settings_box_menu_item_add (GimpSettingsBox *box,
                                  const gchar     *label,
                                  GCallback        callback)
 {
-  GtkWidget *item;
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GtkWidget              *item;
 
   if (label)
     {
@@ -516,7 +549,7 @@ gimp_settings_box_menu_item_add (GimpSettingsBox *box,
       item = gtk_separator_menu_item_new ();
     }
 
-  gtk_menu_shell_append (GTK_MENU_SHELL (box->menu), item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (private->menu), item);
   gtk_widget_show (item);
 
   return item;
@@ -530,7 +563,7 @@ gimp_settings_box_row_separator_func (GtkTreeModel *model,
   gchar *name = NULL;
 
   gtk_tree_model_get (model, iter,
-                      GIMP_CONTAINER_COMBO_BOX_COLUMN_NAME, &name,
+                      GIMP_CONTAINER_TREE_STORE_COLUMN_NAME, &name,
                       -1);
   g_free (name);
 
@@ -543,10 +576,23 @@ gimp_settings_box_setting_selected (GimpContainerView *view,
                                     gpointer           insert_data,
                                     GimpSettingsBox   *box)
 {
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+
   if (object)
     {
       gimp_config_copy (GIMP_CONFIG (object),
-                        GIMP_CONFIG (box->config), 0);
+                        GIMP_CONFIG (private->config), 0);
+
+      /*  reset the "time" property, otherwise explicitly storing the
+       *  config as setting will also copy the time, and the stored
+       *  object will be considered to be among the automatically
+       *  stored recently used settings
+       */
+      if (g_object_class_find_property (G_OBJECT_GET_CLASS (private->config),
+                                        "time"))
+        {
+          g_object_set (private->config, "time", 0, NULL);
+        }
 
       gimp_container_view_select_item (view, NULL);
     }
@@ -567,9 +613,11 @@ gimp_settings_box_menu_press (GtkWidget       *widget,
                               GdkEventButton  *bevent,
                               GimpSettingsBox *box)
 {
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+
   if (bevent->type == GDK_BUTTON_PRESS)
     {
-      gtk_menu_popup (GTK_MENU (box->menu),
+      gtk_menu_popup (GTK_MENU (private->menu),
                       NULL, NULL,
                       gimp_settings_box_menu_position, widget,
                       bevent->button, bevent->time);
@@ -599,58 +647,65 @@ static void
 gimp_settings_box_import_activate (GtkWidget       *widget,
                                    GimpSettingsBox *box)
 {
-  gimp_settings_box_file_dialog (box, box->import_dialog_title, FALSE);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+
+  gimp_settings_box_file_dialog (box, private->import_dialog_title, FALSE);
 }
 
 static void
 gimp_settings_box_export_activate (GtkWidget       *widget,
                                    GimpSettingsBox *box)
 {
-  gimp_settings_box_file_dialog (box, box->export_dialog_title, TRUE);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+
+  gimp_settings_box_file_dialog (box, private->export_dialog_title, TRUE);
 }
 
 static void
 gimp_settings_box_manage_activate (GtkWidget       *widget,
                                    GimpSettingsBox *box)
 {
-  GtkWidget *toplevel;
-  GtkWidget *editor;
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GtkWidget              *toplevel;
+  GtkWidget              *editor;
+  GtkWidget              *content_area;
 
-  if (box->editor_dialog)
+  if (private->editor_dialog)
     {
-      gtk_window_present (GTK_WINDOW (box->editor_dialog));
+      gtk_window_present (GTK_WINDOW (private->editor_dialog));
       return;
     }
 
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (box));
 
-  box->editor_dialog = gimp_dialog_new (_("Manage Saved Settings"),
-                                        "gimp-settings-editor-dialog",
-                                        toplevel, 0,
-                                        NULL, NULL,
-                                        GTK_STOCK_CLOSE,
-                                        GTK_RESPONSE_CLOSE,
-                                        NULL);
+  private->editor_dialog = gimp_dialog_new (_("Manage Saved Settings"),
+                                            "gimp-settings-editor-dialog",
+                                            toplevel, 0,
+                                            NULL, NULL,
+                                            GTK_STOCK_CLOSE,
+                                            GTK_RESPONSE_CLOSE,
+                                            NULL);
 
-  g_object_add_weak_pointer (G_OBJECT (box->editor_dialog),
-                             (gpointer) &box->editor_dialog);
+  g_object_add_weak_pointer (G_OBJECT (private->editor_dialog),
+                             (gpointer) &private->editor_dialog);
   g_signal_connect (toplevel, "unmap",
                     G_CALLBACK (gimp_settings_box_toplevel_unmap),
-                    box->editor_dialog);
+                    private->editor_dialog);
 
-  g_signal_connect (box->editor_dialog, "response",
+  g_signal_connect (private->editor_dialog, "response",
                     G_CALLBACK (gimp_settings_box_manage_response),
                     box);
 
-  editor = gimp_settings_editor_new (box->gimp,
-                                     box->config,
-                                     box->container);
+  editor = gimp_settings_editor_new (private->gimp,
+                                     private->config,
+                                     private->container);
   gtk_container_set_border_width (GTK_CONTAINER (editor), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (box->editor_dialog)->vbox),
-                     editor);
+
+  content_area = gtk_dialog_get_content_area (GTK_DIALOG (private->editor_dialog));
+  gtk_box_pack_start (GTK_BOX (content_area), editor, TRUE, TRUE, 0);
   gtk_widget_show (editor);
 
-  gtk_widget_show (box->editor_dialog);
+  gtk_widget_show (private->editor_dialog);
 }
 
 static void
@@ -658,12 +713,13 @@ gimp_settings_box_favorite_callback (GtkWidget   *query_box,
                                      const gchar *string,
                                      gpointer     data)
 {
-  GimpSettingsBox *box = GIMP_SETTINGS_BOX (data);
-  GimpConfig      *config;
+  GimpSettingsBox        *box     = GIMP_SETTINGS_BOX (data);
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GimpConfig             *config;
 
-  config = gimp_config_duplicate (GIMP_CONFIG (box->config));
+  config = gimp_config_duplicate (GIMP_CONFIG (private->config));
   gimp_object_set_name (GIMP_OBJECT (config), string);
-  gimp_container_add (box->container, GIMP_OBJECT (config));
+  gimp_container_add (private->container, GIMP_OBJECT (config));
   g_object_unref (config);
 
   gimp_settings_box_serialize (box);
@@ -674,23 +730,24 @@ gimp_settings_box_file_dialog (GimpSettingsBox *box,
                                const gchar     *title,
                                gboolean         save)
 {
-  GtkWidget *toplevel;
-  GtkWidget *dialog;
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GtkWidget              *toplevel;
+  GtkWidget              *dialog;
 
-  if (box->file_dialog)
+  if (private->file_dialog)
     {
-      gtk_window_present (GTK_WINDOW (box->file_dialog));
+      gtk_window_present (GTK_WINDOW (private->file_dialog));
       return;
     }
 
   if (save)
-    gtk_widget_set_sensitive (box->import_item, FALSE);
+    gtk_widget_set_sensitive (private->import_item, FALSE);
   else
-    gtk_widget_set_sensitive (box->export_item, FALSE);
+    gtk_widget_set_sensitive (private->export_item, FALSE);
 
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (box));
 
-  box->file_dialog = dialog =
+  private->file_dialog = dialog =
     gtk_file_chooser_dialog_new (title, GTK_WINDOW (toplevel),
                                  save ?
                                  GTK_FILE_CHOOSER_ACTION_SAVE :
@@ -712,7 +769,7 @@ gimp_settings_box_file_dialog (GimpSettingsBox *box,
   gtk_window_set_role (GTK_WINDOW (dialog), "gimp-import-export-settings");
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 
-  g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer) &box->file_dialog);
+  g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer) &private->file_dialog);
   g_signal_connect (toplevel, "unmap",
                     G_CALLBACK (gimp_settings_box_toplevel_unmap),
                     dialog);
@@ -731,34 +788,34 @@ gimp_settings_box_file_dialog (GimpSettingsBox *box,
                     G_CALLBACK (gtk_true),
                     NULL);
 
-  if (box->default_folder &&
-      g_file_test (box->default_folder, G_FILE_TEST_IS_DIR))
+  if (private->default_folder &&
+      g_file_test (private->default_folder, G_FILE_TEST_IS_DIR))
     {
       gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (dialog),
-                                            box->default_folder, NULL);
+                                            private->default_folder, NULL);
 
-      if (! box->last_filename)
+      if (! private->last_filename)
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
-                                             box->default_folder);
+                                             private->default_folder);
     }
-  else if (! box->last_filename)
+  else if (! private->last_filename)
     {
       gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
                                            g_get_home_dir ());
     }
 
-  if (box->last_filename)
+  if (private->last_filename)
     gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog),
-                                   box->last_filename);
+                                   private->last_filename);
 
-  gimp_help_connect (box->file_dialog, gimp_standard_help_func,
-                     box->file_dialog_help_id, NULL);
+  gimp_help_connect (private->file_dialog, gimp_standard_help_func,
+                     private->file_dialog_help_id, NULL);
 
   /*  allow callbacks to add widgets to the dialog  */
   g_signal_emit (box, settings_box_signals[FILE_DIALOG_SETUP], 0,
-                 box->file_dialog, save);
+                 private->file_dialog, save);
 
-  gtk_widget_show (box->file_dialog);
+  gtk_widget_show (private->file_dialog);
 }
 
 static void
@@ -766,8 +823,11 @@ gimp_settings_box_file_response (GtkWidget       *dialog,
                                  gint             response_id,
                                  GimpSettingsBox *box)
 {
-  GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (box));
-  gboolean   save;
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GtkWidget              *toplevel;
+  gboolean                save;
+
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (box));
 
   if (toplevel)
     g_signal_handlers_disconnect_by_func (toplevel,
@@ -792,17 +852,17 @@ gimp_settings_box_file_response (GtkWidget       *dialog,
 
       if (success)
         {
-          g_free (box->last_filename);
-          box->last_filename = g_strdup (filename);
+          g_free (private->last_filename);
+          private->last_filename = g_strdup (filename);
         }
 
       g_free (filename);
     }
 
   if (save)
-    gtk_widget_set_sensitive (box->import_item, TRUE);
+    gtk_widget_set_sensitive (private->import_item, TRUE);
   else
-    gtk_widget_set_sensitive (box->export_item, TRUE);
+    gtk_widget_set_sensitive (private->export_item, TRUE);
 
   gtk_widget_destroy (dialog);
 }
@@ -829,6 +889,40 @@ gimp_settings_box_toplevel_unmap (GtkWidget *toplevel,
   gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_DELETE_EVENT);
 }
 
+static void
+gimp_settings_box_truncate_list (GimpSettingsBox *box,
+                                 gint             max_recent)
+{
+  GimpSettingsBoxPrivate *private = GET_PRIVATE (box);
+  GList                  *list;
+  gint                    n_recent = 0;
+
+  list = GIMP_LIST (private->container)->list;
+  while (list)
+    {
+      GimpConfig *config = list->data;
+      guint       t;
+
+      list = g_list_next (list);
+
+      g_object_get (config,
+                    "time", &t,
+                    NULL);
+
+      if (t > 0)
+        {
+          n_recent++;
+
+          if (n_recent > max_recent)
+            gimp_container_remove (private->container, GIMP_OBJECT (config));
+        }
+      else
+        {
+          break;
+        }
+    }
+}
+
 
 /*  public functions  */
 
@@ -843,7 +937,8 @@ gimp_settings_box_new (Gimp          *gimp,
                        const gchar   *default_folder,
                        const gchar   *last_filename)
 {
-  GimpSettingsBox *box;
+  GimpSettingsBox        *box;
+  GimpSettingsBoxPrivate *private;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (GIMP_IS_CONFIG (config), NULL);
@@ -857,24 +952,32 @@ gimp_settings_box_new (Gimp          *gimp,
                       "filename",  filename,
                       NULL);
 
-  box->import_dialog_title = g_strdup (import_dialog_title);
-  box->export_dialog_title = g_strdup (export_dialog_title);
-  box->file_dialog_help_id = g_strdup (file_dialog_help_id);
-  box->default_folder      = g_strdup (default_folder);
-  box->last_filename       = g_strdup (last_filename);
+  private = GET_PRIVATE (box);
+
+  private->import_dialog_title = g_strdup (import_dialog_title);
+  private->export_dialog_title = g_strdup (export_dialog_title);
+  private->file_dialog_help_id = g_strdup (file_dialog_help_id);
+  private->default_folder      = g_strdup (default_folder);
+  private->last_filename       = g_strdup (last_filename);
 
   return GTK_WIDGET (box);
 }
 
 void
-gimp_settings_box_add_current (GimpSettingsBox *box)
+gimp_settings_box_add_current (GimpSettingsBox *box,
+                               gint             max_recent)
 {
-  GimpConfig *config = NULL;
-  GList      *list;
+  GimpSettingsBoxPrivate *private;
+  GimpConfig             *config = NULL;
+  GList                  *list;
 
   g_return_if_fail (GIMP_IS_SETTINGS_BOX (box));
 
-  for (list = GIMP_LIST (box->container)->list; list; list = g_list_next (list))
+  private = GET_PRIVATE (box);
+
+  for (list = GIMP_LIST (private->container)->list;
+       list;
+       list = g_list_next (list))
     {
       guint t;
 
@@ -884,7 +987,8 @@ gimp_settings_box_add_current (GimpSettingsBox *box)
                     "time", &t,
                     NULL);
 
-      if (t > 0 && gimp_config_is_equal_to (config, GIMP_CONFIG (box->config)))
+      if (t > 0 && gimp_config_is_equal_to (config,
+                                            GIMP_CONFIG (private->config)))
         {
           g_object_set (config,
                         "time", (guint) time (NULL),
@@ -895,14 +999,24 @@ gimp_settings_box_add_current (GimpSettingsBox *box)
 
   if (! list)
     {
-      config = gimp_config_duplicate (GIMP_CONFIG (box->config));
+      config = gimp_config_duplicate (GIMP_CONFIG (private->config));
       g_object_set (config,
                     "time", (guint) time (NULL),
                     NULL);
 
-      gimp_container_insert (box->container, GIMP_OBJECT (config), 0);
+      gimp_container_insert (private->container, GIMP_OBJECT (config), 0);
       g_object_unref (config);
     }
 
+  gimp_settings_box_truncate_list (box, max_recent);
+
   gimp_settings_box_serialize (box);
 }
+
+GtkWidget *
+gimp_settings_box_get_combo (GimpSettingsBox *box)
+{
+  g_return_val_if_fail (GIMP_IS_SETTINGS_BOX (box), NULL);
+
+  return GET_PRIVATE (box)->combo;
+}
