@@ -4,10 +4,10 @@
  * gimputils.c
  * Copyright (C) 2003  Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -33,6 +32,15 @@
 #include "libgimp/libgimp-intl.h"
 
 
+/**
+ * SECTION: gimputils
+ * @title: gimputils
+ * @short_description: Utilities of general interest
+ *
+ * Utilities of general interest
+ **/
+
+
 /**
  * gimp_utf8_strtrim:
  * @str: an UTF-8 encoded string (or %NULL)
@@ -109,7 +117,7 @@ gimp_utf8_strtrim (const gchar *str,
  * @warning_format: The message format for the warning message if conversion
  *                  to UTF-8 fails. See the <function>printf()</function>
  *                  documentation.
- * @Varargs:        The parameters to insert into the format string.
+ * @...:            The parameters to insert into the format string.
  *
  * This function takes any string (UTF-8 or not) and always returns a valid
  * UTF-8 string.
@@ -237,7 +245,7 @@ gimp_filename_to_utf8 (const gchar *filename)
  * In some languages, mnemonics are handled by adding the mnemonic
  * character in brackets (like "File (_F)"). This function recognizes
  * this construct and removes the whole bracket construction to get
- * rid of the mnemonic (see bug #157561).
+ * rid of the mnemonic (see bug 157561).
  *
  * Return value: A (possibly stripped) copy of @str which should be
  *               freed using g_free() when it is not needed any longer.
@@ -417,13 +425,13 @@ gimp_enum_get_desc (GEnumClass *enum_class,
  * @value:      an integer value
  * @value_name: return location for the value's name (or %NULL)
  * @value_nick: return location for the value's nick (or %NULL)
- * @value_desc: return location for the value's translated desc (or %NULL)
+ * @value_desc: return location for the value's translated description (or %NULL)
  * @value_help: return location for the value's translated help (or %NULL)
  *
  * Checks if @value is valid for the enum registered as @enum_type.
  * If the value exists in that enum, its name, nick and its translated
- * desc and help are returned (if @value_name, @value_nick, @value_desc
- * and @value_help are not %NULL).
+ * description and help are returned (if @value_name, @value_nick,
+ * @value_desc and @value_help are not %NULL).
  *
  * Return value: %TRUE if @value is valid for the @enum_type,
  *               %FALSE otherwise
@@ -461,19 +469,36 @@ gimp_enum_get_value (GType         enum_type,
 
           enum_desc = gimp_enum_get_desc (enum_class, value);
 
-          if (value_desc) {
-            *value_desc = ((enum_desc && enum_desc->value_desc) ?
-                           g_strip_context (enum_desc->value_desc,
-                                            dgettext (gimp_type_get_translation_domain (enum_type),
-                                                      enum_desc->value_desc)) :
-                           NULL);
-          }
+          if (value_desc)
+            {
+              if (enum_desc && enum_desc->value_desc)
+                {
+                  const gchar *context;
+
+                  context = gimp_type_get_translation_context (enum_type);
+
+                  if (context)  /*  the new way, using NC_()    */
+                    *value_desc = g_dpgettext2 (gimp_type_get_translation_domain (enum_type),
+                                                context,
+                                                enum_desc->value_desc);
+                  else          /*  for backward compatibility  */
+                    *value_desc = g_strip_context (enum_desc->value_desc,
+                                                   dgettext (gimp_type_get_translation_domain (enum_type),
+                                                             enum_desc->value_desc));
+                }
+              else
+                {
+                  *value_desc = NULL;
+                }
+            }
 
           if (value_help)
-            *value_help = ((enum_desc && enum_desc->value_desc) ?
-                           dgettext (gimp_type_get_translation_domain (enum_type),
-                                     enum_desc->value_help) :
-                           NULL);
+            {
+              *value_help = ((enum_desc && enum_desc->value_help) ?
+                             dgettext (gimp_type_get_translation_domain (enum_type),
+                                       enum_desc->value_help) :
+                             NULL);
+            }
         }
 
       success = TRUE;
@@ -489,9 +514,9 @@ gimp_enum_get_value (GType         enum_type,
  * @enum_class: a #GEnumClass
  * @enum_value: a #GEnumValue from @enum_class
  *
- * Retrieves the translated desc for a given @enum_value.
+ * Retrieves the translated description for a given @enum_value.
  *
- * Return value: the translated desc of the enum value
+ * Return value: the translated description of the enum value
  *
  * Since: GIMP 2.2
  **/
@@ -505,9 +530,20 @@ gimp_enum_value_get_desc (GEnumClass *enum_class,
   enum_desc = gimp_enum_get_desc (enum_class, enum_value->value);
 
   if (enum_desc && enum_desc->value_desc)
-    return g_strip_context (enum_desc->value_desc,
-                            dgettext (gimp_type_get_translation_domain (type),
-                                      enum_desc->value_desc));
+    {
+      const gchar *context;
+
+      context = gimp_type_get_translation_context (type);
+
+      if (context)  /*  the new way, using NC_()    */
+        return g_dpgettext2 (gimp_type_get_translation_domain (type),
+                             context,
+                             enum_desc->value_desc);
+      else          /*  for backward compatibility  */
+        return g_strip_context (enum_desc->value_desc,
+                                dgettext (gimp_type_get_translation_domain (type),
+                                          enum_desc->value_desc));
+    }
 
   return enum_value->value_name;
 }
@@ -581,13 +617,13 @@ gimp_flags_get_first_desc (GFlagsClass *flags_class,
  * @value:      an integer value
  * @value_name: return location for the value's name (or %NULL)
  * @value_nick: return location for the value's nick (or %NULL)
- * @value_desc: return location for the value's translated desc (or %NULL)
+ * @value_desc: return location for the value's translated description (or %NULL)
  * @value_help: return location for the value's translated help (or %NULL)
  *
  * Checks if @value is valid for the flags registered as @flags_type.
- * If the value exists in that flags, its name, nick and its translated
- * desc and help are returned (if @value_name, @value_nick, @value_desc
- * and @value_help are not %NULL).
+ * If the value exists in that flags, its name, nick and its
+ * translated description and help are returned (if @value_name,
+ * @value_nick, @value_desc and @value_help are not %NULL).
  *
  * Return value: %TRUE if @value is valid for the @flags_type,
  *               %FALSE otherwise
@@ -648,9 +684,9 @@ gimp_flags_get_first_value (GType         flags_type,
  * @flags_class: a #GFlagsClass
  * @flags_value: a #GFlagsValue from @flags_class
  *
- * Retrieves the translated desc for a given @flags_value.
+ * Retrieves the translated description for a given @flags_value.
  *
- * Return value: the translated desc of the flags value
+ * Return value: the translated description of the flags value
  *
  * Since: GIMP 2.2
  **/
