@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -52,7 +51,7 @@
 #define CDISPLAY_IS_LCMS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CDISPLAY_TYPE_LCMS))
 
 
-typedef struct _CdisplayLcms CdisplayLcms;
+typedef struct _CdisplayLcms      CdisplayLcms;
 typedef struct _CdisplayLcmsClass CdisplayLcmsClass;
 
 struct _CdisplayLcms
@@ -68,32 +67,28 @@ struct _CdisplayLcmsClass
 };
 
 
-GType               cdisplay_lcms_get_type     (void);
+GType               cdisplay_lcms_get_type             (void);
 
-static void         cdisplay_lcms_finalize     (GObject           *object);
+static void         cdisplay_lcms_finalize             (GObject           *object);
 
-static GtkWidget  * cdisplay_lcms_configure    (GimpColorDisplay  *display);
-static void         cdisplay_lcms_convert      (GimpColorDisplay  *display,
-                                                guchar            *buf,
-                                                gint               width,
-                                                gint               height,
-                                                gint               bpp,
-                                                gint               bpl);
-static void         cdisplay_lcms_changed      (GimpColorDisplay  *display);
+static GtkWidget  * cdisplay_lcms_configure            (GimpColorDisplay  *display);
+static void         cdisplay_lcms_convert_surface      (GimpColorDisplay  *display,
+                                                        cairo_surface_t   *surface);
+static void         cdisplay_lcms_changed              (GimpColorDisplay  *display);
 
-static cmsHPROFILE  cdisplay_lcms_get_rgb_profile      (CdisplayLcms *lcms);
-static cmsHPROFILE  cdisplay_lcms_get_display_profile  (CdisplayLcms *lcms);
-static cmsHPROFILE  cdisplay_lcms_get_printer_profile  (CdisplayLcms *lcms);
+static cmsHPROFILE  cdisplay_lcms_get_rgb_profile      (CdisplayLcms      *lcms);
+static cmsHPROFILE  cdisplay_lcms_get_display_profile  (CdisplayLcms      *lcms);
+static cmsHPROFILE  cdisplay_lcms_get_printer_profile  (CdisplayLcms      *lcms);
 
-static void         cdisplay_lcms_attach_labelled      (GtkTable     *table,
-                                                        gint          row,
-                                                        const gchar  *text,
-                                                        GtkWidget    *widget);
-static void         cdisplay_lcms_update_profile_label (CdisplayLcms *lcms,
-                                                        const gchar  *name);
-static void         cdisplay_lcms_notify_profile       (GObject      *config,
-                                                        GParamSpec   *pspec,
-                                                        CdisplayLcms *lcms);
+static void         cdisplay_lcms_attach_labelled      (GtkTable          *table,
+                                                        gint               row,
+                                                        const gchar       *text,
+                                                        GtkWidget         *widget);
+static void         cdisplay_lcms_update_profile_label (CdisplayLcms      *lcms,
+                                                        const gchar       *name);
+static void         cdisplay_lcms_notify_profile       (GObject           *config,
+                                                        GParamSpec        *pspec,
+                                                        CdisplayLcms      *lcms);
 
 
 static const GimpModuleInfo cdisplay_lcms_info =
@@ -129,15 +124,15 @@ cdisplay_lcms_class_init (CdisplayLcmsClass *klass)
   GObjectClass          *object_class  = G_OBJECT_CLASS (klass);
   GimpColorDisplayClass *display_class = GIMP_COLOR_DISPLAY_CLASS (klass);
 
-  object_class->finalize = cdisplay_lcms_finalize;
+  object_class->finalize         = cdisplay_lcms_finalize;
 
-  display_class->name        = _("Color Management");
-  display_class->help_id     = "gimp-colordisplay-lcms";
-  display_class->stock_id    = GIMP_STOCK_DISPLAY_FILTER_LCMS;
+  display_class->name            = _("Color Management");
+  display_class->help_id         = "gimp-colordisplay-lcms";
+  display_class->stock_id        = GIMP_STOCK_DISPLAY_FILTER_LCMS;
 
-  display_class->configure   = cdisplay_lcms_configure;
-  display_class->convert     = cdisplay_lcms_convert;
-  display_class->changed     = cdisplay_lcms_changed;
+  display_class->configure       = cdisplay_lcms_configure;
+  display_class->convert_surface = cdisplay_lcms_convert_surface;
+  display_class->changed         = cdisplay_lcms_changed;
 
   cmsErrorAction (LCMS_ERROR_IGNORE);
 }
@@ -207,7 +202,7 @@ cdisplay_lcms_configure (GimpColorDisplay *display)
   if (! config)
     return NULL;
 
-  vbox = gtk_vbox_new (FALSE, 12);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
 
   hint = gimp_hint_box_new (_("This filter takes its configuration "
                               "from the Color Management section "
@@ -258,24 +253,55 @@ cdisplay_lcms_configure (GimpColorDisplay *display)
 }
 
 static void
-cdisplay_lcms_convert (GimpColorDisplay *display,
-                       guchar           *buf,
-                       gint              width,
-                       gint              height,
-                       gint              bpp,
-                       gint              bpl)
+cdisplay_lcms_convert_surface (GimpColorDisplay *display,
+                               cairo_surface_t  *surface)
 {
-  CdisplayLcms *lcms = CDISPLAY_LCMS (display);
-  gint          y;
+  CdisplayLcms   *lcms   = CDISPLAY_LCMS (display);
+  gint            width  = cairo_image_surface_get_width (surface);
+  gint            height = cairo_image_surface_get_height (surface);
+  gint            stride = cairo_image_surface_get_stride (surface);
+  guchar         *buf    = cairo_image_surface_get_data (surface);
+  cairo_format_t  fmt    = cairo_image_surface_get_format (surface);
+  guchar         *rowbuf;
+  gint            x, y;
+  guchar          r, g, b, a;
 
-  if (bpp != 3)
+  if (fmt != CAIRO_FORMAT_ARGB32)
     return;
 
   if (! lcms->transform)
     return;
 
-  for (y = 0; y < height; y++, buf += bpl)
-    cmsDoTransform (lcms->transform, buf, buf, width);
+  rowbuf = g_malloc (stride);
+
+  for (y = 0; y < height; y++, buf += stride)
+    {
+      /* Switch buf from ARGB premul to ARGB non-premul, since lcms ignores the
+       * alpha channel.  The macro takes care of byte order.
+       */
+      for (x = 0; x < width; x++)
+        {
+          GIMP_CAIRO_ARGB32_GET_PIXEL (buf + 4*x, r, g, b, a);
+          rowbuf[4*x+0] = a;
+          rowbuf[4*x+1] = r;
+          rowbuf[4*x+2] = g;
+          rowbuf[4*x+3] = b;
+        }
+
+      cmsDoTransform (lcms->transform, rowbuf, rowbuf, width);
+
+      /* And back to ARGB premul */
+      for (x = 0; x < width; x++)
+        {
+          a = rowbuf[4*x+0];
+          r = rowbuf[4*x+1];
+          g = rowbuf[4*x+2];
+          b = rowbuf[4*x+3];
+          GIMP_CAIRO_ARGB32_SET_PIXEL (buf + 4*x, r, g, b, a);
+        }
+    }
+
+  g_free (rowbuf);
 }
 
 static void
@@ -340,8 +366,8 @@ cdisplay_lcms_changed (GimpColorDisplay *display)
           cmsSetAlarmCodes (r, g, b);
         }
 
-      lcms->transform = cmsCreateProofingTransform (src_profile,  TYPE_RGB_8,
-                                                    dest_profile, TYPE_RGB_8,
+      lcms->transform = cmsCreateProofingTransform (src_profile, TYPE_ARGB_8,
+                                                    dest_profile, TYPE_ARGB_8,
                                                     proof_profile,
                                                     config->simulation_intent,
                                                     config->display_intent,
@@ -356,8 +382,8 @@ cdisplay_lcms_changed (GimpColorDisplay *display)
       if (! dest_profile)
        dest_profile = cmsCreate_sRGBProfile ();
 
-      lcms->transform = cmsCreateTransform (src_profile,  TYPE_RGB_8,
-                                            dest_profile, TYPE_RGB_8,
+      lcms->transform = cmsCreateTransform (src_profile, TYPE_ARGB_8,
+                                            dest_profile, TYPE_ARGB_8,
                                             config->display_intent,
                                             flags);
     }
@@ -427,11 +453,12 @@ cdisplay_lcms_get_screen (CdisplayLcms *lcms,
 
   g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
 
-  if (GTK_IS_WIDGET (managed) && GTK_WIDGET_DRAWABLE (managed))
+  if (GTK_IS_WIDGET (managed) && gtk_widget_is_drawable (GTK_WIDGET (managed)))
     {
       GtkWidget *widget = GTK_WIDGET (managed);
 
-      *monitor = gdk_screen_get_monitor_at_window (screen, widget->window);
+      *monitor = gdk_screen_get_monitor_at_window (screen,
+                                                   gtk_widget_get_window (widget));
     }
   else
     {
@@ -442,53 +469,6 @@ cdisplay_lcms_get_screen (CdisplayLcms *lcms,
 }
 
 
-#ifdef GDK_WINDOWING_QUARTZ
-
-typedef struct
-{
-  guchar *data;
-  gsize   len;
-} ProfileTransfer;
-
-enum
-{
-  openReadSpool  = 1,  /* start read data process         */
-  openWriteSpool = 2,  /* start write data process        */
-  readSpool      = 3,  /* read specified number of bytes  */
-  writeSpool     = 4,  /* write specified number of bytes */
-  closeSpool     = 5   /* complete data transfer process  */
-};
-
-static OSErr
-lcms_cdisplay_lcms_flatten_profile (SInt32  command,
-                                    SInt32 *size,
-                                    void   *data,
-                                    void   *refCon)
-{
-  ProfileTransfer *transfer = refCon;
-
-  switch (command)
-    {
-    case openWriteSpool:
-      g_return_val_if_fail (transfer->data == NULL && transfer->len == 0, -1);
-      break;
-
-    case writeSpool:
-      transfer->data = g_realloc (transfer->data, transfer->len + *size);
-      memcpy (transfer->data + transfer->len, data, *size);
-      transfer->len += *size;
-      break;
-
-    default:
-      break;
-    }
-
-  return 0;
-}
-
-#endif /* GDK_WINDOWING_QUARTZ */
-
-
 static cmsHPROFILE
 cdisplay_lcms_get_display_profile (CdisplayLcms *lcms)
 {
@@ -540,18 +520,26 @@ cdisplay_lcms_get_display_profile (CdisplayLcms *lcms)
 
       if (prof)
         {
-          ProfileTransfer transfer = { NULL, 0 };
-          Boolean         foo;
+          CFDataRef data;
 
-          CMFlattenProfile (prof, 0,
-                            lcms_cdisplay_lcms_flatten_profile, &transfer,
-                            &foo);
+          data = CMProfileCopyICCData (NULL, prof);
           CMCloseProfile (prof);
 
-          if (transfer.data)
+          if (data)
             {
-              profile = cmsOpenProfileFromMem (transfer.data, transfer.len);
-              g_free (transfer.data);
+              UInt8 *buffer = g_malloc (CFDataGetLength (data));
+
+              /* We cannot use CFDataGetBytesPtr(), because that returns
+               * a const pointer where cmsOpenProfileFromMem wants a
+               * non-const pointer.
+               */
+              CFDataGetBytes (data, CFRangeMake (0, CFDataGetLength (data)),
+                              buffer);
+
+              profile = cmsOpenProfileFromMem (buffer, CFDataGetLength (data));
+
+              g_free (buffer);
+              CFRelease (data);
             }
         }
     }
