@@ -4,9 +4,9 @@
  * Color management plug-in based on littleCMS
  * Copyright (C) 2006, 2007  Sven Neumann <sven@gimp.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -34,6 +33,7 @@
 
 
 #define PLUG_IN_BINARY          "lcms"
+#define PLUG_IN_ROLE            "gimp-lcms"
 
 #define PLUG_IN_PROC_SET        "plug-in-icc-profile-set"
 #define PLUG_IN_PROC_SET_RGB    "plug-in-icc-profile-set-rgb"
@@ -151,18 +151,18 @@ static GimpPDBStatusType  lcms_dialog            (GimpColorConfig *config,
 
 static const GimpParamDef set_args[] =
 {
-  { GIMP_PDB_INT32,  "run-mode",     "Interactive, non-interactive"     },
+  { GIMP_PDB_INT32,  "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"     },
   { GIMP_PDB_IMAGE,  "image",        "Input image"                      },
   { GIMP_PDB_STRING, "profile",      "Filename of an ICC color profile" }
 };
 static const GimpParamDef set_rgb_args[] =
 {
-  { GIMP_PDB_INT32,  "run-mode",     "Interactive, non-interactive"     },
+  { GIMP_PDB_INT32,  "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"     },
   { GIMP_PDB_IMAGE,  "image",        "Input image"                      },
 };
 static const GimpParamDef apply_args[] =
 {
-  { GIMP_PDB_INT32,  "run-mode",     "Interactive, non-interactive"     },
+  { GIMP_PDB_INT32,  "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"     },
   { GIMP_PDB_IMAGE,  "image",        "Input image"                      },
   { GIMP_PDB_STRING, "profile",      "Filename of an ICC color profile" },
   { GIMP_PDB_INT32,  "intent",       "Rendering intent (enum GimpColorRenderingIntent)" },
@@ -170,7 +170,7 @@ static const GimpParamDef apply_args[] =
 };
 static const GimpParamDef apply_rgb_args[] =
 {
-  { GIMP_PDB_INT32,  "run-mode",     "Interactive, non-interactive"     },
+  { GIMP_PDB_INT32,  "run-mode",     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"     },
   { GIMP_PDB_IMAGE,  "image",        "Input image"                      },
   { GIMP_PDB_INT32,  "intent",       "Rendering intent (enum GimpColorRenderingIntent)" },
   { GIMP_PDB_INT32,  "bpc",          "Black point compensation"         }
@@ -757,7 +757,7 @@ lcms_image_get_profile (GimpColorConfig *config,
 
   g_return_val_if_fail (image != -1, NULL);
 
-  parasite = gimp_image_parasite_find (image, "icc-profile");
+  parasite = gimp_image_get_parasite (image, "icc-profile");
 
   if (parasite)
     {
@@ -804,7 +804,7 @@ lcms_image_set_profile (gint32       image,
 
       if (! file)
         {
-          g_message (error->message);
+          g_message ("%s", error->message);
           g_error_free (error);
 
           return FALSE;
@@ -837,9 +837,9 @@ lcms_image_set_profile (gint32       image,
                                     g_mapped_file_get_length (file),
                                     g_mapped_file_get_contents (file));
 
-      g_mapped_file_free (file);
+      g_mapped_file_unref (file);
 
-      gimp_image_parasite_attach (image, parasite);
+      gimp_image_attach_parasite (image, parasite);
       gimp_parasite_free (parasite);
     }
   else
@@ -847,10 +847,10 @@ lcms_image_set_profile (gint32       image,
       if (undo_group)
         gimp_image_undo_group_start (image);
 
-      gimp_image_parasite_detach (image, "icc-profile");
+      gimp_image_detach_parasite (image, "icc-profile");
     }
 
-  gimp_image_parasite_detach (image, "icc-profile-name");
+  gimp_image_detach_parasite (image, "icc-profile-name");
 
   if (undo_group)
     gimp_image_undo_group_end (image);
@@ -916,7 +916,7 @@ lcms_image_apply_profile (gint32                    image,
 
   if (saved_selection != -1)
     {
-      gimp_selection_load (saved_selection);
+      gimp_image_select_item (image, GIMP_CHANNEL_OP_REPLACE, saved_selection);
       gimp_image_remove_channel (image, saved_selection);
     }
 
@@ -1004,9 +1004,9 @@ lcms_image_transform_indexed (gint32                    image,
 {
   cmsHTRANSFORM   transform;
   guchar         *cmap;
-  gint            num_colors;
+  gint            n_cmap_bytes;
 
-  cmap = gimp_image_get_colormap (image, &num_colors);
+  cmap = gimp_image_get_colormap (image, &n_cmap_bytes);
 
   transform = cmsCreateTransform (src_profile,  TYPE_RGB_8,
                                   dest_profile, TYPE_RGB_8,
@@ -1015,15 +1015,15 @@ lcms_image_transform_indexed (gint32                    image,
 
   if (transform)
     {
-      cmsDoTransform (transform, cmap, cmap, num_colors);
-      cmsDeleteTransform(transform);
+      cmsDoTransform (transform, cmap, cmap, n_cmap_bytes / 3);
+      cmsDeleteTransform (transform);
     }
   else
     {
       g_warning ("cmsCreateTransform() failed!");
     }
 
-  gimp_image_set_colormap (image, cmap, num_colors);
+  gimp_image_set_colormap (image, cmap, n_cmap_bytes);
 }
 
 static void
@@ -1109,7 +1109,7 @@ lcms_load_profile (const gchar *filename,
 
   if (! file)
     {
-      g_message (error->message);
+      g_message ("%s", error->message);
       g_error_free (error);
 
       return NULL;
@@ -1130,7 +1130,7 @@ lcms_load_profile (const gchar *filename,
                  gimp_filename_to_utf8 (filename));
     }
 
-  g_mapped_file_free (file);
+  g_mapped_file_unref (file);
 
   return profile;
 }
@@ -1145,7 +1145,7 @@ lcms_icc_profile_src_label_new (gint32       image,
   gchar     *desc;
   gchar     *text;
 
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
   name = gimp_image_get_name (image);
   text = g_strdup_printf (_("The image '%s' has an embedded color profile:"),
@@ -1224,7 +1224,7 @@ lcms_icc_apply_dialog (gint32       image,
   gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
   dialog = gimp_dialog_new (_("Convert to RGB working space?"),
-                            PLUG_IN_BINARY,
+                            PLUG_IN_ROLE,
                             NULL, 0,
                             gimp_standard_help_func, PLUG_IN_PROC_APPLY,
 
@@ -1245,9 +1245,10 @@ lcms_icc_apply_dialog (gint32       image,
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  vbox = gtk_vbox_new (FALSE, 12);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
   label = lcms_icc_profile_src_label_new (image, src_profile);
@@ -1451,7 +1452,7 @@ lcms_dialog (GimpColorConfig *config,
   dialog = gimp_dialog_new (apply ?
                             _("Convert to ICC Color Profile") :
                             _("Assign ICC Color Profile"),
-                            PLUG_IN_BINARY,
+                            PLUG_IN_ROLE,
                             NULL, 0,
                             gimp_standard_help_func,
                             apply ? PLUG_IN_PROC_APPLY : PLUG_IN_PROC_SET,
@@ -1470,9 +1471,10 @@ lcms_dialog (GimpColorConfig *config,
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  main_vbox = gtk_vbox_new (FALSE, 12);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
   frame = gimp_frame_new (_("Current Color Profile"));
@@ -1506,11 +1508,11 @@ lcms_dialog (GimpColorConfig *config,
       GtkWidget *hbox;
       GtkWidget *toggle;
 
-      vbox = gtk_vbox_new (FALSE, 6);
+      vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
       gtk_box_pack_start (GTK_BOX (main_vbox), vbox, FALSE, FALSE, 0);
       gtk_widget_show (vbox);
 
-      hbox = gtk_hbox_new (FALSE, 6);
+      hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
       gtk_widget_show (hbox);
 
